var router = require('express').Router();
var db = require('./database');
var ev = require('./events');
var defines = require('./defines');
var pointer = require('json-pointer');
var _ = require('lodash');
var moment = require('moment');

// Routes declaration

router.get('/config', function (req, res) {
    const clientIp = Object.values(require("os").networkInterfaces())
        .flat()
        .filter((item) => !item.internal && item.family === "IPv4")
        .find(Boolean).address;

    res.json({
        'pullpointUrl': 'ws://' + clientIp + ':' + defines.listen_port + '/ws/events',
    });
});

router.get('/health', function (req, res) {
    res.json({
        'status': 'The connector is up and running'
    });
});

router.get('/events', function (req, res) {

    // TODO rename (/event ?)

    if (!req.query.eventid)
        return res.sendStatus(400);
    
    var event = db.get('events')
        .find({eventId: req.query.eventid})
        .value();

    // Returns an HTML page containing the event details or 404 error if the
    // event doesn't exist
    if (event) {
        res.render('eventDetail', {event: event, label: ev.labels[event.type]});
    } else {
        res.sendStatus(404);
    }
});

router.get('/events/labels', function (req, res) {
    res.json({
        'labels': ev.labels,
        'descriptions': ev.descriptions,
        'params': ev.params
    });
});

router.get('/events/search/criteria', function (req, res) {
    res.json({
        'criteria': [
            {
                'name': 'type',
                'description': 'Event type',
                'format': 'enum',
                'values': [
                    'analytics.motion',
                    'demo.event1',
                    'demo.event2'
                ],
                "texts": [
                    'Motion detected',
                    'Demo event 1',
                    'Demo event 2'
                ]
            },
            {
                'name': 'timestamp',
                'description': 'Time of the event',
                'format': 'datetime'
            }
        ]
    });
});

var validCriteria = {
    type: {format: 'enum', values: ['analytics.motion', 'demo.event1', 'demo.event2']},
    timestamp: {format: 'datetime'},
    customProperty: {format: 'integer'}
};

function checkCriterion(c) {
    if (!c.name || _.isUndefined(validCriteria[c.name]) ||
        _.isUndefined(c.value))
        return false;

    if (validCriteria[c.name].format == 'enum' &&
        validCriteria[c.name].values.indexOf(c.value) == -1)
        return false;

    return true;
}

router.post('/events/purge', function (req, res) {
    if (!req.body.purgeBefore || !moment(req.body.purgeBefore, moment.ISO_8601).isValid())
        return res.sendStatus(400);
    ev.purgeEvents(req.body.purgeBefore);
    res.json({});
});

router.post('/events/search', function (req, res) {
    // Getting limit and offset
    var limit = 25;
    var offset = 0;

    if (req.body.paging) {
        if (_.isInteger(req.body.paging.offset))
            offset = req.body.paging.offset;

        if (_.isInteger(req.body.paging.limit))
            limit = req.body.paging.limit;
    }

    // Getting search criteria
    if (!_.isArray(req.body.criteria))
        return res.sendStatus(400);

    var predicateList = [];
    var badInput = false;

    // Checking criteria and building predicate list
    _.forEach(req.body.criteria, function (c) {
        if (!checkCriterion(c)) {
            badInput = true;
            return;
        }

        var tmpPred = null;

        if (_.isUndefined(c.comparator) ||
            c.comparator == '=') {
            tmpPred = function (event) { return (event[c.name] == c.value); };
        } else if (c.comparator == '!=') {
            tmpPred = function (event) { return (event[c.name] != c.value); };
        } else if (c.comparator == '>=') {
            tmpPred = function (event) { return (event[c.name] >= c.value); };
        } else if (c.comparator == '<=') {
            tmpPred = function (event) { return (event[c.name] <= c.value); };
        } else if (c.comparator == '>') {
            tmpPred = function (event) { return (event[c.name] > c.value); };
        } else if (c.comparator == '<') {
            tmpPred = function (event) { return (event[c.name] < c.value); };
        } else if (c.comparator == 'LIKE') {
            tmpPred = function (event) { return (event[c.name].indexOf(c.value) != -1); };
        } else {
            badInput = true;
            return;
        }

        predicateList.push(tmpPred);
    });

    if (badInput)
        return res.sendStatus(400);

    var predicate = function (event) {
        var result = true;

        _.forEach(predicateList, function (fct) {
            if (fct(event) == false)
                result = false;
        });

        return result;
    };

    // Request the database for matching events
    var events = db.get('events')
        .filter(predicate)
        .sortBy(function (o) { return o.timestamp; })
        .reverse()
        .slice(offset, offset + limit)
        .reverse()
        .value().map(function (e) {
            if (ev.descriptions[e.type]) {
                e['label'] = ev.descriptions[e.type];
                if (e.description) {
                    var objPointer = pointer(e.description);
                    var founds = ev.descriptions[e.type].matchAll('%\{([0-9a-zA-Z/]*)\}');
                    for (const found of founds) {
                        if (objPointer.has(found[1]))
                            e.label = e.label.replace(found[0],objPointer.get(found[1])); 
                    }
                }
            } else 
                e['label'] = e.type;
            return e;
        })

    res.json({
        'events': events,
        'paging': {
            'next': {
                'offset': offset + limit,
                'limit': limit
            }
        }
    });
});

module.exports = router;
